/*
 * This file is distributed according to the GNU Lesser Public Licence.
 * Created on 13 avr. 2005
 *
 */
package imageDragAndDrop;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.TransferHandler;

/**
 * @author Serge Rosmorduc
 */
public class ImagePanel extends JPanel {

    public ImagePanel() {
        setPreferredSize(new Dimension(200, 200));
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
     */
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.drawOval(100, 100, 70, 70);
        setTransferHandler(new MyTransfertHandler());
        addMouseListener(new DragListener());
    }

    /**
     * @return
     */
    private Image getImage() {
        System.out.println("calling getImage");
        Image result;
        result = new BufferedImage(getWidth(), getHeight(),
                BufferedImage.TYPE_INT_RGB);
        Graphics g = result.getGraphics();
        paintComponent(g);
        g.dispose();
        return result;
    }

    class ImageTransfert implements Transferable {

        /*
         * (non-Javadoc)
         * 
         * @see java.awt.datatransfer.Transferable#getTransferDataFlavors()
         */
        public DataFlavor[] getTransferDataFlavors() {
            DataFlavor[] result = new DataFlavor[2];
            //result[0] = DataFlavor.imageFlavor;
            result[0]= new DataFlavor("image/jpeg","jpeg");
            //result[1]= new DataFlavor("image/x-java-image;class=java.awt.Image", null);
            result[1] = DataFlavor.stringFlavor;
            return result;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.awt.datatransfer.Transferable#isDataFlavorSupported(java.awt.datatransfer.DataFlavor)
         */
        public boolean isDataFlavorSupported(DataFlavor flavor) {
            System.out.println("is "+ flavor+ " supported ?");
            return false;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.awt.datatransfer.Transferable#getTransferData(java.awt.datatransfer.DataFlavor)
         */
        public Object getTransferData(DataFlavor flavor)
                throws UnsupportedFlavorException, IOException {
            System.out.println("asking for "+ flavor);
            Object result = null;
            if (DataFlavor.imageFlavor.equals(flavor)) {
                result = getImage();
            } else if (DataFlavor.stringFlavor.equals(flavor)) {
                result = ":-)";
            } else {
                throw new UnsupportedFlavorException(flavor);
            }
            return result;
        }
    }

    class MyTransfertHandler extends TransferHandler {

        ImageIcon icon = null;

        /*
         * (non-Javadoc)
         * 
         * @see javax.swing.TransferHandler#createTransferable(javax.swing.JComponent)
         */
        protected Transferable createTransferable(JComponent c) {
            return new ImageTransfert();
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.swing.TransferHandler#getSourceActions(javax.swing.JComponent)
         */
        public int getSourceActions(JComponent c) {
            return TransferHandler.COPY;
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.swing.TransferHandler#getVisualRepresentation(java.awt.datatransfer.Transferable)
         */
        public Icon getVisualRepresentation(Transferable t) {
            System.out.println("asking for icon");
            if (icon == null) {
                BufferedImage img = new BufferedImage(30, 30,
                        BufferedImage.TYPE_INT_RGB);
                Graphics g = img.getGraphics();
                g.setColor(Color.RED);
                g.fillRect(0, 0, 30, 30);
                g.dispose();
                ImageIcon icon = new ImageIcon(img);
            }
            return icon;
        }

    }
    
    class DragListener extends MouseAdapter {
        /* (non-Javadoc)
         * @see java.awt.event.MouseAdapter#mousePressed(java.awt.event.MouseEvent)
         */
        /*public void mousePressed(MouseEvent e) {        	
            getTransferHandler().exportAsDrag(ImagePanel.this, e,
                    TransferHandler.COPY);
            //e.consume();
        }*/
        
        /* (non-Javadoc)
		 * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
		 */
		public void mouseClicked(MouseEvent e) {
			System.out.println("clicks :"+ e.getClickCount());
			if (e.getClickCount() == 2) {
				System.out.println("copy");
				Toolkit.getDefaultToolkit().getSystemClipboard().setContents(new ImagePanel.ImageTransfert(), null);
			}
		}
    }

    public static void main(String[] args) {
        JFrame f = new JFrame("image d&d");
        f.getContentPane().add(new ImagePanel());
        f.pack();
        f.addWindowListener(new WindowAdapter() {
           /* (non-Javadoc)
         * @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent)
         */
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        } 
        });
        f.setVisible(true);
    }
}
